# include <stdio.h>
# include <math.h>
# include <stdlib.h>
# include <vector>
using namespace std;
typedef vector<double> Data;

double f(Data &x)
{
    int n = x.size();
    double sum = 0.0;
    for(int i=0;i<n;i++)
    {
        sum+=(pow(x[i],4.0)-16.0*pow(x[i],2.0)+5.0*x[i]);
    }
    return 0.5*sum;
}


Data gradient(Data &x)
{
    Data g;
    g.resize(x.size());
    int n = x.size();
    for(int i=0;i<n;i++)
    {
        g[i]=0.5*(4.0*pow(x[i],3.0)-32.0*x[i]+5.0);
    }
    return g;
}

Data finiteGradient(Data &x)
{
    Data g;
    g.resize(x.size());
    double fx = f(x);
    int n = x.size();
    const double eps =1e-6;
    for(int i=0;i<n;i++)
    {
        x[i]+=eps;
        double f1=f(x);
        g[i]=(f1-fx)/eps;
        x[i]-=eps;
    }
    return g;
}

double distance(Data &x1,Data &x2)
{
    double sum=0.0;
    for(int i=0;i<x1.size();i++)
        sum+=pow(x1[i]-x2[i],2.0);
    return sqrt(sum);
}

int main()
{
    for(int n=2;n<=20;n+=2)
    {
        Data xpoint;
        xpoint.resize(n);
        srand(1);
        const double a= -5.0;
        const double b=  5.0;
        const int npoints=10000;
        double error = 0.0;
        for(int i=1;i<=npoints;i++)
        {
            for(int j=0;j<n;j++)
                xpoint[j]=a+(b-a)*rand()*1.0/RAND_MAX;
            Data g1 = gradient(xpoint);
            Data g2 = finiteGradient(xpoint);
            error+=distance(g1,g2);
        }
        printf("Error for dimension %4d is %20.10lg\n",n,error);
    }
    return 0;
}
